/* decompress.c  */
/* Uses a previously built nodetree to complete Huffman decompression.*/

/* Author: Zach Tomaszewski */
/* Date: 10 Mar 2002        */


#include "huffdcmp.h"


/*
 Takes: a readable compressed .huf file, 
        the root of a node tree for that file,
        and a file to output to.
 Decompresses the input file, printing the results to the output file.
*/

Node* decompress(FILE* fileIn, Node* tree, FILE* fileOut){
  int byte1 = fgetc(fileIn);
  int byte2 = fgetc(fileIn);
  int byte3 = fgetc(fileIn);

  while (byte3 != EOF) {
    process(byte1, 8, tree, fileOut);
    byte1 = byte2;
    byte2 = byte3;
    byte3 = fgetc(fileIn);
  } 
  /*hit end of file, so process last two bytes
    where last byte in file tells the number of bits to process
    of the second-to-last byte*/
  process(byte1, byte2, tree, fileOut);

}


/*
 Takes a byte,
   the number of bits to process in that byte,
   a pointer to a Huffman tree root node,
   and the file to output to.
 Decompresses the given number of bits in the byte, using the tree,
   and outputs to the output file.
*/
void process(char byte, int bits, Node* tree, FILE* fileOut){
  int i;
  char bit;
  for (i=1; i<=bits; i++){
     //shift over and mask to grab the ith bit from the ith
     bit = (byte >> (8 - i)) & 1;
     traverse(bit, tree, fileOut);
  }
}


/*
 Takes a bit (0 or 1),
   a pointer to Huffman tree root node,
   and a file to print to.
 Goes either left (0) or right (1) from the current position
 in the tree.  If doing so ends in a leaf node, prints the value
 of that node to the file and returns to the root position.  
 The function remembers the  current location in the tree; 
 it will reset to the root if given a different tree.
 reset to root if given a different tree.
*/
void traverse(char bit, Node* tree, FILE* fileOut) {
  static Node* root;
  static Node* current;

  if (root != tree) { //new tree
    root = tree;
    current = root;
  }

  if (bit == 0) { //go left
    current = (*current).left;
  }else { //go right
    current = (*current).right;
  }

  if ( (*current).left == NULL) { //leaf node, so print and reset
    fputc((*current).content, fileOut);
    current = root;
  }
}
