/* nodeTree.c  */
/* Builds a tree of nodes used in Huffman decompression.   */

/* Author: Zach Tomaszewski */
/* Date: 10 Mar 2002        */


#include "huffdcmp.h"
#include<stdlib.h>

#define MAXSTACK 30
Node* stack[MAXSTACK];
unsigned int stackCount;


/*
 Takes a pointer to an open and readable binary tree file.
 Builds a tree of Node.
 Returns a point to the root node.
*/
Node* readTree(FILE* fileIn){

   int nextChar;
   Node* left;
   Node* right;

   while ( (nextChar = getc(fileIn)) != EOF ){
     if (nextChar != '*') { // a leaf node
        if (nextChar == '\\') { //escaped character
           nextChar = getc(fileIn);
        }
        push(makeNode(nextChar, NULL, NULL));
     }else {  // an internal node
        nextChar = 0; //not used in internal nodes
        right = pop();
        left = pop();
        push(makeNode(nextChar, left, right));
     }
   }
   return pop();
}


/*
 Builds and returns a pointer to a node with the given contents.
   Returns NULL if memory can not be allocted.
*/
Node* makeNode(int content, Node* left, Node* right){
  Node* newNode;
  newNode = malloc(sizeof(Node));
  if (newNode != NULL) {
    (*newNode).left = left;
    (*newNode).right = right;
    (*newNode).content = content;
  }
  return newNode;
}



/*
 Adds a node to the top of the stack
*/
void push(Node* node){
  if (stackCount >= MAXSTACK) {
    puts("Error: Node stack overflowed while building the tree!");
    exit(1);
  }else {
    stackCount++;  //move up one space
    stack[stackCount - 1] = node;  //but index == count-1.
  }
}

/*
 Removes and returns the top node of the stack.
 Returns NULL if stack is empty.
*/
Node* pop(){
  if (stackCount == 0) {
    return NULL;
  }else {
    return stack[--stackCount]; //index == count- 1
  }
}



/*
  Prints a node tree to the screen.
  (Code by Wes Peterson.) 
*/
void printtree(Node* r)
{
    if(r==NULL) return;
    printf("%08x  ", r);  /* location of node  */
    if(r->left==NULL)  /* leaf node  */
        printf("(%02x)   ", r->content);
    else printf("Int.   ");  /* internal node  */
    printf("left = %08x  right = %08x\n",
        r->left,   r->right); /* The two pointers */
    printtree(r->left);  /* print the left subtree*/
    printtree(r->right); /* print the right subtree*/
}


/*
 Recursive function to return the number of leaves contained
  by a given node.
*/
int countLeaves(Node* n){
  if ((*n).left == NULL && (*n).right==NULL) { //leaf node
    return 1;
  }else {
    return (countLeaves((*n).left) + countLeaves((*n).right));
  }
}


/*
 Recursive function to return the largest byte value in any of the
  nodes within this tree.
*/
int largestValue(Node* n){
  int left = 0;
  int right = 0;
  int largest = (*n).content;
  if ((*n).left != NULL) {
    left = largestValue((*n).left);
  }
  if ((*n).right != NULL) {
    right = largestValue((*n).right);
  }
  largest = (left > largest) ? left : largest;
  largest = (right > largest) ? right : largest;
  return largest;
}

